@@ -9,7 +9,8 @@ from .diem_account import DiemAccount
 from .models import PaymentUri, Subaddress, Account, Transaction, Event, KycSample, Payment, PaymentCommand
 from .event_puller import EventPuller
 from .json_input import JsonInput
-from .... import jsonrpc, offchain, utils, LocalAccount, identifier
+from ... import LocalAccount
+from .... import jsonrpc, offchain, utils, identifier
 from ....offchain import KycDataObject, Status, AbortCode, CommandResponseObject
 import threading, logging, numpy
 
@@ -21,7 +22,7 @@ class Base:
         self.store = InMemoryStore()
         self.offchain = offchain.Client(account.account_address, client, account.hrp)
         self.kyc_sample: KycSample = KycSample.gen(name)
-        self.event_puller = EventPuller(client=client, store=self.store, hrp=account.hrp)
+        self.event_puller = EventPuller(client=client, store=self.store, hrp=account.hrp, logger=logger)
         self.event_puller.add(account.account_address)
         self.event_puller.head()
 
@@ -39,7 +40,8 @@ class Base:
 
     def _validate_account_identifier(self, name: str, val: str) -> None:
         try:
-            self.diem_account.account.decode_account_identifier(val)
+            account_address, _ = self.diem_account.account.decode_account_identifier(val)
+            self.diem_account.client.must_get_account(account_address)
         except ValueError as e:
             raise ValueError("%r is invalid account identifier: %s" % (name, e))
 
@@ -82,14 +84,17 @@ class Base:
 
 class OffChainAPI(Base):
     def offchain_api(self, request_sender_address: str, request_bytes: bytes) -> CommandResponseObject:
-        cmd = self.offchain.process_inbound_request(request_sender_address, request_bytes)
-        getattr(self, "_handle_offchain_%s" % utils.to_snake(cmd))(cmd)
-        return offchain.reply_request(cid=cmd.id())
+        request = self.offchain.deserialize_inbound_request(request_sender_address, request_bytes)
+        getattr(self, "_handle_offchain_%s" % utils.to_snake(request.command_type))(request_sender_address, request)
+        return offchain.reply_request(cid=request.cid)
 
     def jws_serialize(self, resp: CommandResponseObject) -> bytes:
         return offchain.jws.serialize(resp, self.diem_account.account.compliance_key.sign)
 
-    def _handle_offchain_payment_command(self, new_offchain_cmd: offchain.PaymentCommand) -> None:
+    def _handle_offchain_payment_command(
+        self, request_sender_address: str, request: offchain.CommandRequestObject
+    ) -> None:
+        new_offchain_cmd = self.offchain.process_inbound_payment_command_request(request_sender_address, request)
         try:
             cmd = self.store.find(PaymentCommand, reference_id=new_offchain_cmd.reference_id())
             if new_offchain_cmd != cmd.to_offchain_command():
@@ -142,7 +147,7 @@ class BackgroundTasks(OffChainAPI):
 
     def _send_pending_payments(self) -> None:
         for txn in self.store.find_all(Transaction, status=Transaction.Status.pending):
-            self.logger.info("processing %s" % txn)
+            self.logger.info("processing %s", txn)
             try:
                 if self.offchain.is_my_account_id(str(txn.payee)):
                     self._send_internal_payment_txn(txn)
@@ -174,12 +179,12 @@ class BackgroundTasks(OffChainAPI):
                 diem_txn = self.diem_account.client.wait_for_transaction(str(txn.signed_transaction), timeout_secs=0.1)
                 self.store.update(txn, status=Transaction.Status.completed, diem_transaction_version=diem_txn.version)
             except jsonrpc.WaitForTransactionTimeout as e:
-                self.store.create_event(txn.account_id, "info", str(e))
+                self.logger.debug("wait for txn(%s) timeout: %s", txn, e)
             except jsonrpc.TransactionHashMismatchError as e:
-                self.store.create_event(txn.account_id, "info", str(e))
+                self.logger.warn("txn(%s) hash mismatched(%s), re-submit", txn, e)
                 self.store.update(txn, signed_transaction=self.diem_account.submit_p2p(txn, self._txn_metadata(txn)))
             except (jsonrpc.TransactionExpired, jsonrpc.TransactionExecutionFailed) as e:
-                self.store.create_event(txn.account_id, "info", str(e))
+                self.logger.error("txn(%s) execution expired / failed(%s), canceled", txn, e)
                 reason = "something went wrong with transaction execution: %s" % e
                 self.store.update(txn, status=Transaction.Status.canceled, cancel_reason=reason)
         else:
@@ -211,9 +216,9 @@ class BackgroundTasks(OffChainAPI):
                     amount=txn.amount,
                     receiver_account_id=str(txn.payee),
                 )
-                self.offchain.send_command(command, self.diem_account.account.compliance_key.sign)
                 self._create_payment_command(txn.account_id, command)
                 self.store.update(txn, reference_id=command.reference_id())
+                self._send_offchain_command(command)
 
     def _process_offchain_commands(self) -> None:
         cmds = self.store.find_all(PaymentCommand, is_inbound=True, is_abort=False, is_ready=False, process_error=None)
@@ -224,12 +229,25 @@ class BackgroundTasks(OffChainAPI):
                 if action:
                     fn = getattr(self, "_offchain_action_%s" % action.value)
                     new_offchain_cmd = fn(cmd.account_id, offchain_cmd)
-                    self.offchain.send_command(new_offchain_cmd, self.diem_account.account.compliance_key.sign)
                     self._update_payment_command(cmd, new_offchain_cmd)
+                    self._send_offchain_command(new_offchain_cmd)
             except Exception as e:
                 self.logger.exception(e)
                 self.store.update(cmd, process_error=str(e))
 
+    def _send_offchain_command(self, command: offchain.Command) -> None:
+        """send offchain command with retries
+
+        We only do limited in process retries, as we are expecting counterparty service
+        should be available in dev / testing environment, and fail immediately if counterparty
+        service is not available.
+        In production environment, you may need implement better retry system for handling
+        counterparty service temporarily not available case.
+        """
+
+        retry = jsonrpc.Retry(5, 0.2, Exception)
+        retry.execute(lambda: self.offchain.send_command(command, self.diem_account.account.compliance_key.sign))
+
     def _offchain_action_evaluate_kyc_data(self, account_id: str, cmd: offchain.PaymentCommand) -> offchain.Command:
         op_kyc_data = cmd.counterparty_actor_obj().kyc_data
         if op_kyc_data is None or self.kyc_sample.match_kyc_data("reject", op_kyc_data):
@@ -239,6 +257,9 @@ class BackgroundTasks(OffChainAPI):
         return self._ready_for_settlement(account_id, cmd)
 
     def _offchain_action_clear_soft_match(self, account_id: str, cmd: offchain.PaymentCommand) -> offchain.Command:
+        account = self.store.find(Account, id=account_id)
+        if account.reject_additional_kyc_data_request:
+            return self._new_reject_kyc_data(cmd, "no need additional KYC data")
         return cmd.new_command(additional_kyc_data="{%r: %r}" % ("account_id", account_id))
 
     def _offchain_action_review_kyc_data(self, account_id: str, cmd: offchain.PaymentCommand) -> offchain.Command:
@@ -265,6 +286,9 @@ class App(BackgroundTasks):
         account = self.store.create(
             Account,
             kyc_data=data.get_nullable("kyc_data", str, self._validate_kyc_data),
+            # reject_additional_kyc_data_request is belongs to mini-wallet stub API, used for
+            # setting up mini-wallet stub to reject additional_kyc_data request.
+            reject_additional_kyc_data_request=data.get_nullable("reject_additional_kyc_data_request", bool),
         )
         balances = data.get_nullable("balances", dict)
         if balances:
@@ -272,7 +296,7 @@ class App(BackgroundTasks):
                 self._create_transaction(
                     account.id, Transaction.Status.completed, JsonInput({"currency": c, "amount": a})
                 )
-        return account
+        return Account(id=account.id)
 
     def create_account_payment(self, account_id: str, data: JsonInput) -> Payment:
         self.store.find(Account, id=account_id)
@@ -287,7 +311,7 @@ class App(BackgroundTasks):
         sub = self._gen_subaddress(account_id)
         diem_acc_id = self.diem_account.account.account_identifier(sub.subaddress_hex)
         uri = identifier.encode_intent(diem_acc_id, currency, amount)
-        return PaymentUri(id=sub.id, account_id=account_id, currency=currency, amount=amount, payment_uri=uri)
+        return PaymentUri(id=sub.id, account_id=account_id, payment_uri=uri)
 
     def get_account_balances(self, account_id: str) -> Dict[str, int]:
         self.store.find(Account, id=account_id)
