@@ -5,6 +5,7 @@ from dataclasses import dataclass, field, asdict, fields
 from enum import Enum
 from typing import Optional, List, Dict, Any
 from .... import identifier, offchain, diem_types
+import json
 
 
 @dataclass
@@ -15,6 +16,7 @@ class Base:
 @dataclass
 class Account(Base):
     kyc_data: Optional[str] = field(default=None)
+    reject_additional_kyc_data_request: Optional[bool] = field(default=False)
 
     def kyc_data_object(self) -> offchain.KycDataObject:
         if self.kyc_data:
@@ -27,8 +29,6 @@ class Account(Base):
 class PaymentUri(Base):
     account_id: str
     payment_uri: str
-    currency: Optional[str] = field(default=None)
-    amount: Optional[int] = field(default=None)
 
     def intent(self, hrp: str) -> identifier.Intent:
         return identifier.decode_intent(self.payment_uri, hrp)
@@ -118,6 +118,9 @@ class Transaction(Base):
     def balance_amount(self) -> int:
         return -self.amount if self.payee else self.amount
 
+    def __str__(self) -> str:
+        return "Transaction %s" % json.dumps(asdict(self), indent=2)
+
 
 @dataclass
 class PaymentCommand(Base):
