@@ -26,9 +26,17 @@ class RestClient:
         return self
 
     def create_account(
-        self, balances: Optional[Dict[str, int]] = None, kyc_data: Optional[str] = None
+        self,
+        balances: Optional[Dict[str, int]] = None,
+        kyc_data: Optional[str] = None,
+        reject_additional_kyc_data_request: Optional[bool] = None,
     ) -> "AccountResource":
-        account = self.create("/accounts", kyc_data=kyc_data, balances=balances)
+        kwargs = {
+            "balances": balances,
+            "kyc_data": kyc_data,
+            "reject_additional_kyc_data_request": reject_additional_kyc_data_request,
+        }
+        account = self.create("/accounts", **{k: v for k, v in kwargs.items() if v})
         return AccountResource(client=self, id=account["id"], kyc_data=account.get("kyc_data", None))
 
     def new_soft_match_kyc_data(self) -> str:
@@ -60,19 +68,31 @@ class RestClient:
 
     def send(self, method: str, path: str, data: Optional[str] = None) -> requests.Response:
         url = "%s/%s" % (self.server_url.rstrip("/"), path.lstrip("/"))
-        self.logger.info("%s %s: %s" % (method, path, data))
+        self.logger.debug("%s %s: %s", method, path, data)
         resp = self.session.request(
             method=method,
             url=url.lower(),
             data=data,
             headers={"Content-Type": "application/json", "User-Agent": jsonrpc.client.USER_AGENT_HTTP_HEADER},
         )
-        self.logger.info("response status code: %s" % resp.status_code)
-        self.logger.info("response body: %s" % resp.text)
+        log_level = logging.DEBUG if resp.status_code < 300 else logging.ERROR
+        self.logger.log(log_level, "%s %s: %s - %s", method, path, data, resp.status_code)
+        self.logger.log(log_level, "response body: \n%s", try_json(resp.text))
         resp.raise_for_status()
         return resp
 
 
+def try_json(text: str) -> str:
+    try:
+        obj = json.loads(text)
+        if isinstance(obj, dict):
+            # pretty print error json stacktrace info
+            return "\n".join(["%s: %s" % (k, v) for k, v in obj.items()])
+        return json.dumps(obj, indent=2)
+    except Exception:
+        return text
+
+
 @dataclass
 class AccountResource:
 
@@ -81,7 +101,7 @@ class AccountResource:
     kyc_data: Optional[str] = field(default=None)
 
     def balance(self, currency: str) -> int:
-        return self._balances().get(currency, 0)
+        return self.balances().get(currency, 0)
 
     def send_payment(self, currency: str, amount: int, payee: str) -> Payment:
         p = self.client.create(self._resources("payment"), payee=payee, currency=currency, amount=amount)
@@ -96,17 +116,20 @@ class AccountResource:
 
     def wait_for_balance(self, currency: str, amount: int) -> None:
         def fn() -> None:
-            assert self.balance(currency) == amount, "currency: %s" % currency
+            assert self.balance(currency) == amount
 
         self.wait_for(fn)
 
+    def find_event(self, event_type: str, start_index: int = 0, **kwargs: Any) -> Optional[Event]:
+        events = [e for e in self.events(start_index) if e.type == event_type]
+        for e in events:
+            if _match(json.loads(e.data), **kwargs):
+                return e
+
     def wait_for_event(self, event_type: str, start_index: int = 0, **kwargs: Any) -> None:
         def fn() -> None:
-            events = [e for e in self.events(start_index) if e.type == event_type]
-            for e in events:
-                if _match(json.loads(e.data), **kwargs):
-                    return
-            assert False, "could not find %s event with %s" % (event_type, kwargs)
+            event = self.find_event(event_type, start_index=start_index, **kwargs)
+            assert event, "could not find %s event with %s" % (event_type, (start_index, kwargs))
 
         self.wait_for(fn)
 
@@ -145,7 +168,7 @@ class AccountResource:
     def info(self, *args: Any, **kwargs: Any) -> None:
         self.client.logger.info(*args, **kwargs)
 
-    def _balances(self) -> Dict[str, int]:
+    def balances(self) -> Dict[str, int]:
         """returns account balances object
 
         should always prefer to use func `balance(currency) -> int`, which returns zero
